//    mkmovie: Convert MythTV NuppelVideo files to DivX .avi files
//    Copyright (C) 2003 Michael J. Pedersen <michael.j.pedersen@verizon.net>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// The following three structs are taken from libs/libmythtv/format.h
// and are copied into here for completeness sake. Aside from them,
// there is no reason to have the mythtv source code for any part of
// mythmkmovie, and this just makes it a little easier.
typedef struct rtfileheader
{
  char finfo[12];
  char version[5];
  int  width;
  int  height;
  int  desiredwidth;
  int  desiredheight;
  char pimode;
  double aspect;
  double fps;
  int videoblocks;
  int audioblocks;
  int textsblocks;
  int keyframedist;
} rtfileheader;

typedef struct rtframeheader
{
   char frametype;
   char comptype;
   char keyframe;
   char filters;
   int  timecode;
   int  packetlength;
} rtframeheader;

typedef struct seektable_entry
{
   long long file_offset;
   int keyframe_number; 
} seektable_entry;

int main (int argc, char **argv) {
	rtfileheader info;
	rtframeheader current;
	seektable_entry lastent;
	int extras = 0;
	FILE *nuv;

	if (argc != 2) {
		printf ("Usage: %s <filename>\n", argv[0]);
		exit(1);
	}
	nuv = fopen(argv[1], "r");
	if (nuv == (FILE*)NULL) {
		printf ("ERROR! Unable to read file!\n");
		exit(1);
	}
	fread(&info, sizeof(info), 1, nuv);
	if (info.keyframedist < 1) {
		printf("ERROR! Corrupted file! Bad Header!\n");
		exit(1);
	}
	if (strncmp(info.finfo, "MythTVVideo",11) != 0) {
		printf("ERROR! Corrupted file! Wrong Format!\n");
		exit(1);
	}
	if (strncmp(info.version, "0.07",4) != 0) {
		printf("ERROR! Corrupted file! Wrong Version!\n");
		exit(1);
	}
	fseek(nuv, 0 - sizeof(seektable_entry), SEEK_END);
	fread(&lastent, sizeof(seektable_entry), 1, nuv);
	fseek(nuv, lastent.file_offset, SEEK_SET);
	do {
		fread(&current, sizeof(rtframeheader), 1, nuv);
		fseek(nuv, current.packetlength, SEEK_CUR);
		++extras;
	} while ((current.frametype != 'Q') && (!feof(nuv)));
	fclose(nuv);

	printf("Total Frames: %d\n", (lastent.keyframe_number * info.keyframedist) + extras);
	return(0);
}
